import { useI18n } from '@/hooks/web/useI18n'
import { Table, TableExpose, TableProps, TableSetProps, TableColumn } from '@/components/Table'
import { ElTable, ElMessageBox, ElMessage } from 'element-plus'
import { ref, watch, unref, nextTick, onMounted, computed } from 'vue'
import { setIndex } from '@/components/Table/src/helper'
import { FormSchema } from '@/components/Form'
import { getTableFilterApi, getWidthByFlagApi, saveTableFilterApi } from '@/api/common'

const { t } = useI18n()

interface UseTableConfig {
  /**
   * 是否初始化的时候请求一次
   */
  immediate?: boolean
  rowKey?: string
  fetchDataApi: () => Promise<{
    list: any[]
    total?: number
    rows?: any[]
  }>
  fetchDelApi?: () => Promise<boolean>
  exportListApi?: (option: any) => Promise<any>
}

export const useTable = (config: UseTableConfig) => {
  const { immediate = true } = config

  const loading = ref(false)
  const currentPage = ref(1)
  const pageSize = ref(10)
  const total = ref(0)
  const dataList = ref<any[]>([])
  const params = ref<any>()

  const paramsObj: any = computed(() => {
    return {
      ...params,
      pageSize: pageSize,
      pageNum: currentPage
    }
  })

  watch(
    () => currentPage.value,
    () => {
      methods.getList()
    }
  )

  watch(
    () => pageSize.value,
    () => {
      // 当前页不为1时，修改页数后会导致多次调用getList方法
      if (unref(currentPage) === 1) {
        methods.getList()
      } else {
        currentPage.value = 1
        methods.getList()
      }
    }
  )

  onMounted(() => {
    if (immediate) {
      methods.getList()
    }
  })

  // Table实例
  const tableRef = ref<typeof Table & TableExpose>()

  // ElTable实例
  const elTableRef = ref<ComponentRef<typeof ElTable>>()

  watch(
    () => tableRef.value?.selectionsListAll,
    () => {
      if (tableRef.value?.selectionsListAll.length === 0) {
        elTableRef.value?.clearSelection()
      }
    }
  )

  const register = (ref: typeof Table & TableExpose, elRef: ComponentRef<typeof ElTable>) => {
    tableRef.value = ref
    elTableRef.value = unref(elRef)
  }

  /**
   * 改变字符串为标签样式
   * @param name
   * @returns {string}
   */
  const str2label = (name: string) => {
    let result = ''
    // 首字母小写执行标签化
    if (!name.charAt(0).match(/[A-Z]+/)) {
      // 根据大写字母进行分组
      const names = name.split(/[A-Z]+/)
      // 循环数组
      names.forEach((item, index) => {
        // 判断index>0时进行添加字母处理
        if (index > 0) {
          // 分组时干掉的字母位置
          let strlength = 0
          // 循环获得字母位置
          for (let i = index; i > 0; i--) {
            strlength += names[i - 1].length
          }
          // 从源字符串中截取字母
          const str = name.substring(strlength, strlength + 1)
          // 将字母添加到相应位置
          names[index] = str + item
          // 组合成相应标签格式
          result += '_' + names[index].toLowerCase()
        } else {
          // 数组第一个不用加下划线
          result = names[index].toLowerCase()
        }
      })
    } else {
      result = name.toLowerCase()
    }
    return result
  }
  // 使用orderByColumn字段排序
  const sortChange = (sort: any) => {
    if (sort.order) {
      if (sort.column.sortBy) {
        if (Array.isArray(sort.column.sortBy)) {
          const list: any = []
          sort.column.sortBy.map((ele) => {
            list.push((ele += ' desc'))
          })
          params.value.orderByColumn =
            sort.order === 'descending' ? list.join(',') : sort.column.sortBy.join(',')
          paramsObj.value.orderByColumn = params.value.orderByColumn
        } else {
          params.value.orderByColumn =
            sort.order === 'descending' ? sort.column.sortBy + ' desc' : sort.column.sortBy
          paramsObj.value.orderByColumn = params.value.orderByColumn
        }
      } else {
        params.value.orderByColumn =
          sort.order === 'descending' ? str2label(sort.prop) + ' desc' : str2label(sort.prop)
        paramsObj.value.orderByColumn = params.value.orderByColumn
      }
    } else {
      params.value.orderByColumn = null
      paramsObj.value.orderByColumn = null
    }

    methods.getList()
  }
  // 嵌套params传参排序
  const sortChangeByParam = (sort) => {
    if (sort.order) {
      if (sort.column.sortBy) {
        if (Array.isArray(sort.column.sortBy)) {
          const list: any = []
          sort.column.sortBy.map((ele) => {
            list.push((ele += ' desc'))
          })
          paramsObj.value.params.param =
            sort.order === 'descending' ? list.join(',') : sort.column.sortBy.join(',')
        } else {
          paramsObj.value.params.param =
            sort.order === 'descending' ? sort.column.sortBy + ' desc' : sort.column.sortBy
        }
      } else {
        paramsObj.value.params.param =
          sort.order === 'descending' ? str2label(sort.prop) + ' desc' : str2label(sort.prop)
      }
    } else {
      paramsObj.value.params.param = null
    }
    methods.getList()
  }

  const getTable = async () => {
    await nextTick()
    const table = unref(tableRef)
    if (!table) {
      console.error('The table is not registered. Please use the register method to register')
    }
    return table
  }

  const methods = {
    /**
     * 获取表单数据
     */
    getList: async () => {
      loading.value = true
      try {
        const res = await config?.fetchDataApi()
        if (res) {
          dataList.value = res.list
          total.value = res.total || 0
          nextTick(() => {
            elTableRef.value?.clearSelection()
            dataList.value?.map((item: any, index: number) => {
              item.sortIndexNumber = unref(paramsObj).noPage
                ? index + 1
                : setIndex(true, index, unref(pageSize), unref(currentPage))
              // 判断是否在选择列表中，存在则将当前dom变为勾选状态
              const d = unref(tableRef.value?.selectionsListAll)?.find(
                (s: any) => String(s[config.rowKey || 'id']) === String(item[config.rowKey || 'id'])
              )
              elTableRef.value?.toggleRowSelection(item, d ? true : false)
            })
          })
        }
      } finally {
        loading.value = false
      }
    },

    /**
     * @description 设置table组件的props
     * @param props table组件的props
     */
    setProps: async (props: TableProps = {}) => {
      const table = await getTable()
      table?.setProps(props)
    },

    /**
     * @description 设置column
     * @param columnProps 需要设置的列
     */
    setColumn: async (columnProps: TableSetProps[]) => {
      const table = await getTable()
      table?.setColumn(columnProps)
    },

    /**
     * @description 新增column
     * @param tableColumn 需要新增数据
     * @param index 在哪里新增
     */
    addColumn: async (tableColumn: TableColumn, index?: number) => {
      const table = await getTable()
      table?.addColumn(tableColumn, index)
    },

    /**
     * @description 删除column
     * @param field 删除哪个数据
     */
    delColumn: async (field: string) => {
      const table = await getTable()
      table?.delColumn(field)
    },

    /**
     * @description 获取ElTable组件的实例
     * @returns ElTable instance
     */
    getElTableExpose: async () => {
      await getTable()
      return unref(elTableRef)
    },

    refresh: () => {
      methods.getList()
    },

    // sortableChange: (e: any) => {
    //   console.log('sortableChange', e)
    //   const { oldIndex, newIndex } = e
    //   dataList.value.splice(newIndex, 0, dataList.value.splice(oldIndex, 1)[0])
    //   // to do something
    // }
    // 删除数据
    delList: async (idsLength: number) => {
      const { fetchDelApi } = config
      if (!fetchDelApi) {
        console.warn('fetchDelApi is undefined')
        return
      }
      if (!idsLength) {
        ElMessage.error('请选择要删除的数据')
        return
      }
      return new Promise((resolve, reject) => {
        ElMessageBox.confirm(t('common.delMessage'), t('common.delWarning'), {
          confirmButtonText: t('common.delOk'),
          cancelButtonText: t('common.delCancel'),
          type: 'warning'
        })
          .then(async () => {
            const res = await fetchDelApi()
            if (res) {
              resolve(res)
              ElMessage.success(t('common.delSuccess'))

              // 计算出临界点
              const current =
                unref(total) % unref(pageSize) === idsLength || unref(pageSize) === 1
                  ? unref(currentPage) > 1
                    ? unref(currentPage) - 1
                    : unref(currentPage)
                  : unref(currentPage)

              currentPage.value = current
              methods.getList()
            }
          })
          .catch((err) => {
            reject(err)
          })
      })
    },
    // 导出
    exportList: async (param) => {
      loading.value = true
      if (unref(paramsObj).noPage) {
        delete unref(paramsObj).pageSize
        delete unref(paramsObj).pageNum
        delete unref(paramsObj).noPage
      }
      if (dataList.value.length === 0) {
        ElMessage.warning('查询无数据, 无需导出数据')
        return
      }
      loading.value = true
      await (config?.exportListApi &&
        config?.exportListApi(param).finally(() => {
          loading.value = false
          elTableRef.value?.clearSelection()
        }))
    },
    // 获取列宽
    getTableWidthList: async (flag: string | number, type = 1) => {
      const params = {
        flag
      }
      return new Promise((resolve, reject) => {
        getWidthByFlagApi(params, type)
          .then((res) => {
            if (res) {
              resolve(res)
            } else {
              reject()
            }
          })
          .catch(() => {
            reject()
          })
      })
    },
    // 获取存储的高级搜索&表头排序
    getTableFilterList: async (pageName: string, userId: string | number, type = 1) => {
      const params = {
        moduleName: pageName,
        userId: userId
      }
      return new Promise((resolve, reject) => {
        getTableFilterApi(params, type)
          .then((res) => {
            if (res) {
              resolve(res)
            } else {
              reject()
            }
          })
          .catch(() => {
            reject()
          })
      })
    },
    // 初始化保存列表字段
    saveTableFilterList: async (
      searchSchema: FormSchema[],
      tableColumns: TableColumn[] | null,
      pageName: string,
      userId: string | number,
      type = 1
    ) => {
      const searchList: any = []
      searchSchema.map((item) => {
        if (!item.hidden) {
          searchList.push({
            field: item.field,
            label: item.label,
            hidden: item.hidden
          })
        }
      })

      const tableHeaderList: any = []
      tableColumns?.map((item, index) => {
        tableHeaderList.push({
          hidden: item.hidden,
          sort: index,
          comment: item.label,
          fieldName: item.field,
          fixed: item.fixed,
          type: item.type
        })
      })
      const params = {
        moduleName: pageName,
        userId: userId,
        searchFieldName: searchList.length ? JSON.stringify(searchList) : '',
        tableHeader: tableHeaderList.length ? JSON.stringify(tableHeaderList) : '',
        sortFieldName: ''
      }
      return new Promise((resolve, reject) => {
        saveTableFilterApi(params, type)
          .then((res) => {
            if (res) {
              resolve(res)
            } else {
              reject()
            }
          })
          .catch(() => {
            reject()
          })
      })
    }
  }

  return {
    tableRegister: register,
    tableMethods: methods,
    sortChange,
    sortChangeByParam,
    tableState: {
      currentPage,
      pageSize,
      total,
      dataList,
      loading
    }
  }
}
