import { useI18n } from '@/hooks/web/useI18n'
import { FormItemRule } from 'element-plus'
import { isNumber } from 'lodash-es'

const { t } = useI18n()
type Callback = (error?: string | Error | undefined) => void
interface LengthRange {
  min: number
  max: number
  message?: string
}

export const useValidator = () => {
  const required = (message = '该项为必填项', type: any = 'string'): FormItemRule => {
    return {
      required: true,
      type: type,
      transform: (value) => {
        if (Array.isArray(value)) {
          return value.length ? value : null
        } else if (isNumber(value)) {
          return String(value)
        } else {
          return value || value === 0 ? value.trim() : ''
        }
      },
      message: message || t('common.required')
    }
  }
  // 长度范围
  const lengthRange = (options: LengthRange): FormItemRule => {
    const { min, max, message } = options

    return {
      min,
      max,
      message: message || t('common.lengthRange', { min, max })
    }
  }
  // 不能有空格
  const notSpace = (message?: string): FormItemRule => {
    return {
      validator: (_, val, callback) => {
        if (val?.indexOf(' ') !== -1) {
          callback(new Error(message || t('common.notSpace')))
        } else {
          callback()
        }
      }
    }
  }

  const notSpecialCharacters = (message?: string): FormItemRule => {
    return {
      validator: (_, val, callback) => {
        if (/[`~!@#$%^&*()_+<>?:"{},.\/;'[\]]/gi.test(val)) {
          callback(new Error(message || t('common.notSpecialCharacters')))
        } else {
          callback()
        }
      }
    }
  }

  const phone = (message?: string): FormItemRule => {
    return {
      validator: (_, val, callback) => {
        if (!val) return callback()
        if (!/^1[3456789]\d{9}$/.test(val)) {
          callback(new Error(message || '请输入正确的手机号码'))
        } else {
          callback()
        }
      }
    }
  }

  const email = (message?: string): FormItemRule => {
    return {
      validator: (_, val, callback) => {
        if (!val) return callback()
        if (!/^(\w-*\.*)+@(\w-?)+(\.\w{2,})+$/.test(val)) {
          callback(new Error(message || '请输入正确的邮箱'))
        } else {
          callback()
        }
      }
    }
  }

  const maxlength = (max: number): FormItemRule => {
    return {
      max,
      message: '长度不能超过' + max + '个字符'
    }
  }

  const check = (message?: string): FormItemRule => {
    return {
      validator: (_, val, callback) => {
        if (!val) {
          callback(new Error(message || t('common.required')))
        } else {
          callback()
        }
      }
    }
  }

  // 不能有空格和回车
  // 注意富文本对于空格的解析&nbsp; 前后是否带有空格，富文本内容便利获取
  const validateTextSpace = (val: any, callback: Callback, message: string) => {
    let num = 0
    const reg = /<p>( &nbsp;| &nbsp;\s+)+<\/p>|<p>(<br>)+<\/p>/g
    const errors: any = []
    while (num < val.length && val !== '') {
      num++
      const k = val.match(reg)
      if (k) {
        val = val.replace(k[0], '')
      }
      if (val === '' || val === null) {
        errors.push(new Error(message || '请不要只输入空格或回车'))
      }
    }
    callback(errors)
  }
  // 不能输入纯有空格
  const notAllSpace = (val: any, callback: Callback, message: string) => {
    if (val && val.length > 0 && val.indexOf(' ') !== -1 && String(val).trim() === '') {
      callback(new Error(message))
    } else {
      callback()
    }
  }

  // 根据仅支持汉字字母数字
  const validateNormalBySearch = (val: any, callback: Callback, message: string) => {
    if (/^[A-Za-z0-9\u4e00-\u9fa5]+$/gi.test(val)) {
      callback()
    } else {
      callback(new Error(message))
    }
  }
  // 两个字符串是否想等
  const isEqual = (val1: string, val2: string, callback: Callback, message: string) => {
    if (val1 === val2) {
      callback()
    } else {
      callback(new Error(message))
    }
  }

  // 以英⽂字母开头，只能包含英⽂字母、数字、下划线
  const validateEnglishAndNumber = (val: any, callback: Callback, message: string) => {
    if (/^[a-z][a-z0-9_]*$/.test(val)) {
      callback()
    } else {
      callback(new Error(message))
    }
  }
  // 以英⽂字母开头，只能包含英⽂字母、数字、下划线、短杠
  const validateEngNumber = (val: any, callback: Callback, message: string) => {
    if (/^[A-Za-z0-9_-]+$/.test(val)) {
      callback()
    } else {
      callback(new Error(message))
    }
  }
  // 密码必须由数字、字母、特殊字符组合,请输入6-16位
  const validatePassword = (val: any, callback: Callback, message: string) => {
    // if (/(?=.*\d)(?=.*[a-zA-Z])(?=.*[^a-zA-Z0-9]).{6,16}/.test(val))
    if (/^(?=.*\d)(?=.*[a-zA-Z])(?=.*[~!@#$%^&*])[\da-zA-Z~!@#$%^&*]{6,16}$/.test(val)) {
      callback()
    } else {
      callback(new Error(message))
    }
  }

  // ========================号码类 ======================

  // 电子邮箱格式(以字母数字开头，中间可以是多个数字字母和-；然后是@符号，后面是字母数字；然后是.符号加2-4个字母结尾)
  const validateEmail = (val: any, callback: Callback, message: string) => {
    if (/^([a-zA-Z]|[0-9])(\w|\-)+@[a-zA-Z0-9]+\.([a-zA-Z]{2,4})$/.test(val)) {
      callback()
    } else {
      callback(new Error(message))
    }
  }
  // 国内电话号码和手机号
  //  *              0511 - 440 5222 || 021 - 8788 8822 -
  const validatePhone = (val: any, callback: Callback, message: string) => {
    if (/^((0\d{2,3}-?\d{7,8})|(1[3465789]\d{9}))$/.test(val)) {
      callback()
    } else {
      callback(new Error(message))
    }
  }
  // ========================数字类 ======================
  /**
   * @description 数字
   * @param {*} message 自定义提示信息。参数可不传递
   */
  const validateNumber = (value, callback, message) => {
    const reg = /^\d+(\.\d+)?$/
    if (value && !reg.test(value)) {
      callback(new Error(message || '请输入数字类型'))
    } else {
      callback()
    }
  }
  /**
   * @description 正整数 - 非零正整数
   * @param {*} message: string 自定义提示信息。参数可不传递
   */

  const validatePositiveInteger = (val: any, callback: Callback, message: string) => {
    const reg = /^[1-9]\d*$/
    if (!reg.test(val) && val) {
      callback(new Error(message || '请输入正整数，如：99'))
    } else {
      callback()
    }
  }

  /**
   * @description n位的正数(固定位数)
   * @param {*} message: string 自定义提示信息。参数可不传递
   * @param {*} count: number 位数
   */
  const validateNumberN = (value, callback, count, message) => {
    const reg = count ? new RegExp(`^\\d{${count}}$`) : new RegExp(`^\\d{10}$`)
    if (!reg.test(value)) {
      callback(new Error(message || (count ? `请输入${count}位的正数` : '请输入正整数')))
    } else {
      callback()
    }
  }

  /**
   * @description m-n位正数
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateNumberMN = (value, callback, begin, end, message) => {
    if (begin && end) {
      const reg = new RegExp(`^\\d{${begin},${end}}$`)
      if (!reg.test(value) && value) {
        callback(
          new Error(
            message || (begin && end ? `请输入${begin} ~ ${end}位的正整数` : '请输入正整数')
          )
        )
      } else {
        callback()
      }
    }
  }

  /**
   * @description 整数m位,小数点后n位
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateFloatMN = (value, callback, m, n, message) => {
    const reg = new RegExp(`^-?\\d{1,${m}}(\\.\\d{1,${n}})?$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `整数${m}位,小数点后${n}位`))
    } else {
      callback()
    }
  }
  /**
   * @description 整数m位,小数点后n位的正数
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateFloatNumberMN = (value, callback, m, n, message) => {
    const reg = new RegExp(`^\\d{1,${m}}(\\.\\d{1,${n}})?$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `整数${m}位,小数点后${n}位`))
    } else {
      callback()
    }
  }

  /**
   * @description N位以内正整数 - 非零正整数
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validatePositiveIntegerN = (value: string, callback: any, n: number, message?: string) => {
    const reg = new RegExp(`^[1-9]\\d{0,${n - 1}}$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `请输入${n}位以内正整数`))
    } else {
      callback()
    }
  }
  /**
   * @description n位以内非负整数，包括0
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateNumberIntN = (value: string, callback: any, n: number, message?: string) => {
    const reg = new RegExp(`^[+]?\\d{1,${n}}$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `请输入${n}位以内非负整数`))
    } else {
      callback()
    }
  }
  /**
   * @description n位以内整数，正整数、负整数，包括0
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateIntNumberN = (value: string, callback: any, n: any, message: any) => {
    const reg = new RegExp(`^-?\\d{1,${n}}$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `请输入${n}位以内整数`))
    } else {
      callback()
    }
  }

  /**
   * @description 验证小数点后最多两位
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateDecimalPoint2 = (value, callback, message) => {
    const reg = /^0\.\d\d$/
    if (!reg.test(value) && value) {
      callback(new Error(message || '请输入纯数字,小数点后最多2位,不能为负数'))
    } else {
      callback()
    }
  }
  /**
   * @description 验证小数点后最多N位
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateDecimalPointN = (value, callback, n, message) => {
    const reg = new RegExp(`^-?\\d+(\\.\\d{1,${n}})?$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `小数点后${n}位`))
    } else {
      callback()
    }
  }
  /**
   * @description 验证小数点后最多N位的非负数
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validatePositivePointN = (value, callback, n, message) => {
    const reg = new RegExp(`^\\d+(\\.\\d{1,${n}})?$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `小数点后${n}位`))
    } else {
      callback()
    }
  }
  /**
   * @description 验证时间格式 12:00
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateTime = (value1, value2, callback, message) => {
    const reg = /^(20|21|22|23|[0-1]\d):[0-5]\d$/
    if (value1 && !value2 && !reg.test(value1)) {
      callback(new Error(message || '请输入正确时间格式hh:mm'))
    } else if (!value1 && value2 && !reg.test(value2)) {
      callback(new Error(message || '请输入正确时间格式hh:mm'))
    } else if (value1 && value2 && (!reg.test(value1) || !reg.test(value2))) {
      callback(new Error(message || '请输入正确时间格式hh:mm'))
    } else {
      callback()
    }
  }

  /**
   * @description 验证时间格式 12:00-14:00(120分钟)
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateTimeLength = (value, callback, message) => {
    const reg =
      /^((20|21|22|23|[0-1]\d):[0-5]\d-(20|21|22|23|[0-1]\d):[0-5]\d)(\((\d+(\.\d+)?)分钟\))$/
    const reg1 =
      /^((20|21|22|23|[0-1]\d):[0-5]\d-(20|21|22|23|[0-1]\d):[0-5]\d)(\s+\((\d+(\.\d+)?)分钟\))$/
    let flag = false
    if (value && !reg.test(value) && !reg1.test(value)) {
      flag = false
      callback(new Error(message || '请输入正确时间格式,例：12:00-14:00 (120分钟)'))
    } else {
      flag = true
      callback()
    }
    return flag
  }

  /**
   * @description 罗马数字
   * @param {*} message: string 自定义提示信息。参数可不传递
   */
  const validateRoman = (value, callback, message) => {
    const reg = new RegExp(`^M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$`)
    if (!reg.test(value) && value) {
      callback(new Error(message || `请输入罗马数字`))
    } else {
      callback()
    }
  }

  return {
    required,
    lengthRange,
    notSpace,
    notSpecialCharacters,
    phone,
    email,
    maxlength,
    check,

    validateTextSpace,
    notAllSpace,
    isEqual,
    validatePassword,
    validateEnglishAndNumber,
    validateEmail,
    validatePhone,
    validateNormalBySearch,
    validateEngNumber,
    validatePositiveInteger,
    validateNumber,
    validateNumberN,
    validateNumberMN,
    validateFloatNumberMN,
    validateFloatMN,
    validatePositiveIntegerN,
    validateNumberIntN,
    validateIntNumberN,
    validateDecimalPoint2,
    validateDecimalPointN,
    validatePositivePointN,
    validateTime,
    validateTimeLength,
    validateRoman
  }
}
