/**
 *
 * @param component 需要注册的组件
 * @param alias 组件别名
 * @returns any
 */
export const withInstall = <T>(component: T, alias?: string) => {
  const comp = component as any
  comp.install = (app: any) => {
    app.component(comp.name || comp.displayName, component)
    if (alias) {
      app.config.globalProperties[alias] = component
    }
  }
  return component as T & Plugin
}

/**
 * @param str 需要转下划线的驼峰字符串
 * @returns 字符串下划线
 */
export const humpToUnderline = (str: string): string => {
  return str.replace(/([A-Z])/g, '-$1').toLowerCase()
}

/**
 * @param str 需要转驼峰的下划线字符串
 * @returns 字符串驼峰
 */
export const underlineToHump = (str: string): string => {
  if (!str) return ''
  return str.replace(/\-(\w)/g, (_, letter: string) => {
    return letter.toUpperCase()
  })
}

/**
 * 驼峰转横杠
 */
export const humpToDash = (str: string): string => {
  return str.replace(/([A-Z])/g, '-$1').toLowerCase()
}

export const setCssVar = (prop: string, val: any, dom = document.documentElement) => {
  dom.style.setProperty(prop, val)
}

export const getCssVar = (prop: string, dom = document.documentElement) => {
  return getComputedStyle(dom).getPropertyValue(prop)
}

/**
 * 查找数组对象的某个下标
 * @param {Array} ary 查找的数组
 * @param {Functon} fn 判断的方法
 */
export const findIndex = <T = Recordable>(ary: Array<T>, fn: Fn): number => {
  if (ary.findIndex) {
    return ary.findIndex(fn)
  }
  let index = -1
  ary.some((item: T, i: number, ary: Array<T>) => {
    const ret: T = fn(item, i, ary)
    if (ret) {
      index = i
      return ret
    }
  })
  return index
}

export const trim = (str: string) => {
  return str.replace(/(^\s*)|(\s*$)/g, '')
}

/**
 * @param {Date | number | string} time 需要转换的时间
 * @param {String} fmt 需要转换的格式 如 yyyy-MM-dd、yyyy-MM-dd HH:mm:ss
 */
export function formatTime(time: Date | number | string, fmt: string) {
  if (!time) return ''
  else {
    const date = new Date(time)
    const o = {
      'M+': date.getMonth() + 1,
      'd+': date.getDate(),
      'H+': date.getHours(),
      'm+': date.getMinutes(),
      's+': date.getSeconds(),
      'q+': Math.floor((date.getMonth() + 3) / 3),
      S: date.getMilliseconds()
    }
    if (/(y+)/.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length))
    }
    for (const k in o) {
      if (new RegExp('(' + k + ')').test(fmt)) {
        fmt = fmt.replace(
          RegExp.$1,
          RegExp.$1.length === 1 ? o[k] : ('00' + o[k]).substr(('' + o[k]).length)
        )
      }
    }
    return fmt
  }
}

/**
 * 生成随机字符串
 */
export function toAnyString() {
  const str: string = 'xxxxx-xxxxx-4xxxx-yxxxx-xxxxx'.replace(/[xy]/g, (c: string) => {
    const r: number = (Math.random() * 16) | 0
    const v: number = c === 'x' ? r : (r & 0x3) | 0x8
    return v.toString()
  })
  return str
}

/**
 * 首字母大写
 */
export function firstUpperCase(str: string) {
  return str.toLowerCase().replace(/( |^)[a-z]/g, (L) => L.toUpperCase())
}

/**
 * 把对象转为formData
 */
export function objToFormData(obj: Recordable) {
  const formData = new FormData()
  Object.keys(obj).forEach((key) => {
    formData.append(key, obj[key])
  })
  return formData
}

export function getFileType(val) {
  if (val.url) {
    const list = val.url?.split('.')
    const type = list[list?.length - 1]
    if (type === 'doc' || type === 'xls' || type === 'pptx' || type === 'ppt') {
      return false
    } else {
      return true
    }
  } else {
    return true
  }
}

// 获取assets静态资源
export const getAssetsFile = (url: string) => {
  return new URL(`../assets/${url}`, import.meta.url).href
}

//1,先将base64转换为blob
export function dataURLtoBlob(dataurl) {
  const arr = dataurl.split(',')
  const mime = arr[0].match(/:(.*?);/)[1]
  const bstr = atob(arr[1])
  let n = bstr.length
  const u8arr = new Uint8Array(n)
  while (n--) {
    u8arr[n] = bstr.charCodeAt(n)
  }
  return new Blob([u8arr], { type: mime })
}
//2,再将blob转换为file
export function blobToFile(theBlob, fileName) {
  theBlob.lastModifiedDate = new Date() // 文件最后的修改日期
  theBlob.name = fileName // 文件名
  return new File([theBlob], fileName, { type: theBlob.type, lastModified: Date.now() })
}

// base64转换文件流
export function base64toFile(base64, name?) {
  // 1.先转为 blob格式  base64是此文件的base64格式
  const blob = dataURLtoBlob(base64)
  // 2，在转为 file类型
  const file = blobToFile(blob, name)
  return file
}
// 获取菜单第一个url
export const getFirstUrl = (arr, path) => {
  if (arr.length === 0) {
    return path
  }
  if (arr[0].link) {
    return arr[0].link
  }
  if (arr[0].path.indexOf('/') != -1 || path == '/') {
    path += arr[0].path
  } else {
    path += '/' + arr[0].path
  }
  if (arr[0].children) {
    return getFirstUrl(arr[0].children, path)
  }

  return path
}
// 下拉选项重组
export const selectionEdit = (data: any, lable: string, value: string, isString = false) => {
  const editData = data.map((item: any) => {
    const val = item ? (isString ? String(item[value]) : item[value]) : ''
    return item
      ? (item = Object.assign(item, {
          label: item ? item[lable] : '',
          value: item ? val : ''
        }))
      : false
  })
  return editData
}
// 空值传参,将参数中的空值转换为null或空字符串
export const emptyTransformat = (list: any, data: any, isString = false) => {
  Object.keys(data).map((key: string) => {
    if (list.includes(key) && !data[key] && data[key] !== 0) {
      data[key] = isString ? '' : null
    }
  })
  return data
}

// 下拉数组中不存在当前值的数据时，添加固定值
export const selectArrayformat = (dataList: any, model: any) => {
  dataList.map((data) => {
    if (model[data.value]) {
      const d = data.list.find((s) => s.value === model[data.value])
      if (!d) {
        data.list.push({
          label: model[data.label],
          value: model[data.value],
          disabled: true
        })
      }
    }
  })
  return dataList
}
